import processing.opengl.*;
import processing.pdf.*;
import java.util.Stack;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

public static final int BORDER_WIDTH = 30;
public final color BACKGROUND_COLOR = color( 255 );
public final color TEXT_COLOR = color( 100 );
public static final int OVER_TOLERANCE = 1;

// build color map
int MAX_NUM_SELECTION = 8;
public static final int UPPER_BOUND_MAX_NUM_SELECTION = 100;

color[] SELECTION_COLOR = { #377db8,
                            #4daf4a,
                            #ff7f00,
                            #984ea3,
                            #E31A1C,
                            #ffff33,
                            #a65628,
                            #f781bf };
                            
/*color[] SELECTION_COLOR = { 
                            #1F78B4, 
                            #A6CEE3,
                            #33A02C,
                            #B2DF8A };*/

PFont _plotFont_8, _plotFont_8b, _plotFont_8eb, _plotFont_12, _plotFont_12b, _plotFont_16; 
PFont _veranda_big, _veranda_small;
int _w, _h;

boolean _print_pdf = false;
int _pdf_counter = 0;

CorrelationView _correlation_view;
int[] _correlation_view_origin;
int _correlation_view_width, _correlation_view_height;

TimeSeriesView _timeseries_view;
int[] _timeseries_view_origin;
int _timeseries_view_width, _timeseries_view_height;

String[] _species;
String[] _timepoints;
String[] _correlations;
String _tree_file;
Pathway _pathway;
boolean _ctrl_pressed = false;

int _y_labels = BORDER_WIDTH*3/2;

void setup()
{
  // to capture the size of the monitor
  _w = displayWidth;
  _h = displayHeight-40;

  // but lets just hardcode this for now
  _w = min( _w, 1200);
  _h = min( _h, 800);

  size( _w, _h/*, OPENGL*/ );
  
  //hint(DISABLE_OPENGL_2X_SMOOTH);
  //hint(ENABLE_OPENGL_4X_SMOOTH); 
  
  _veranda_small = createFont( "Verdana", 10, true );
  _veranda_big = createFont( "Verdana", 30, true );
  _plotFont_12 = createFont( "Verdana", 12, true );
  _plotFont_12b = createFont( "Verdana-Bold", 12, true );
  _plotFont_8 = createFont( "PFTempestaSeven", 8, false );
  _plotFont_8b = createFont( "PFTempestaSeven-Bold", 8, false );
  
  /*_veranda_small = loadFont( "Verdana-10.vlw"); 
  _veranda_big = loadFont( "Verdana-30.vlw" );
  _plotFont_12 = loadFont( "Verdana-12.vlw" );
  _plotFont_8 = loadFont( "PFTempestaSeven-8.vlw" );
  _plotFont_8b = loadFont( "PFTempestaSeven-Bold-8.vlw" );*/

  // read in all of the data
  _tree_file = readConfigFile( "./config.txt" );

  // generate the data views
  createViews();
  
  frame.setResizable(true);
  frame.addComponentListener(new ComponentAdapter() {
   public void componentResized(ComponentEvent e) {
     if(e.getSource()==frame) 
     { 
       _w = frame.getWidth();
       _h = frame.getHeight();
       frame.setSize(_w,_h); 
       _h -= 22; // offset on mac
       setDimensions();
     }
   }
  } );
}

void createViews()
{
  _correlation_view_origin = new int[2];
  _timeseries_view_origin = new int[2];
  
  _correlation_view_origin[0] = BORDER_WIDTH;
  _correlation_view_origin[1] = 3*BORDER_WIDTH;
  
  _correlation_view = new CorrelationView();
  _timeseries_view = new TimeSeriesView( _tree_file );
  
  setDimensions();
}

void setDimensions()
{
  _correlation_view_width = 200;
  _correlation_view_height = _h - 4*BORDER_WIDTH;
  
  _timeseries_view_width = _w - _correlation_view_width  - 3*BORDER_WIDTH;
  _timeseries_view_height = _correlation_view_height;
  
  _timeseries_view_origin[0] = _correlation_view_origin[0] + _correlation_view_width + 2*BORDER_WIDTH;
  _timeseries_view_origin[1] = _correlation_view_origin[1];
  
  _correlation_view.setDimensions( _correlation_view_width, _correlation_view_height );
  _timeseries_view.setDimensions( _timeseries_view_width, _timeseries_view_height );
}

void draw() 
{  
  // this is for rendering to a pdf file
  if ( _print_pdf ) {
    String filename = "output_" + nf(_pdf_counter++,2) + ".pdf";
    beginRecord(PDF, filename);
    //textFont(plotFont); 
  }

  background( BACKGROUND_COLOR ); 

  // translate to the origin of the correlation view
  pushMatrix();
  translate( _correlation_view_origin[0], _correlation_view_origin[1] );
  _correlation_view.render();
  popMatrix();
  
  // separate the views with a line
  /*strokeWeight(1);
  stroke( color(150) );
  int x = _correlation_view_origin[0]+_correlation_view_width+BORDER_WIDTH/2;
  x += (_timeseries_view_origin[0]-x)/2;
  line( x, 0, x, _h );*/
  
  // translate to the origin of the timeseries view
  pushMatrix();
  translate( _timeseries_view_origin[0], _timeseries_view_origin[1] );
  _timeseries_view.render();
  popMatrix();
  
  // render the title bar
  int x = BORDER_WIDTH;
  int y = _y_labels+10;
  
  textFont(_veranda_big);
  fill( LABEL_COLOR );
  textAlign( LEFT, BOTTOM );
  text( "PATHLINE", x, y );
  strokeWeight( 2 );
  stroke( LABEL_COLOR );
  float l = textWidth("PATHLINE");
  line( x, y-38, x+l, y-38 );
  
  textFont(_veranda_small);
  text( "v1", x+l, y-4 );
  
  x = _w-x;
  fill( LABEL_COLOR2 );
  textAlign( RIGHT, BOTTOM );
  text( "A TOOL FOR COMPARATIVE FUNCTIONAL GENOMICS", x, y-2 );
  
  x = 185;
  y -= 16;
  /*textFont(_veranda_small);
  textAlign( LEFT, BOTTOM );
  text( "v1.0", x, y );*/
  
  //-----------------------------------------------------------------------
  // render the labels
  textFont(_veranda_small);
  fill( LABEL_COLOR );
  textAlign( CENTER, TOP );
  text( "PATHWAY", _correlation_view_origin[0]+_name_center_x, _y_labels+10 );
  text( "METRIC OVERVIEW", _correlation_view_origin[0]+_pathway_center_x, _y_labels+10 );
  text( "SPECIES", _timeseries_view_origin[0]+_tree_center_x, _y_labels+10 );
  textAlign( LEFT, TOP );
  text( "CURVEMAP", _timeseries_view_origin[0]+_shapemaps_x, _y_labels+10 );
  textAlign( LEFT, TOP );
  if ( _overview_center_x != -1 ) text( "OVERLAYS", _timeseries_view_origin[0]+_overview_center_x, _y_labels+10 );

  // for turning off rendering to a pdf file
  if ( _print_pdf ) {
    _print_pdf = false;
    endRecord();
  }
} 

void mouseMoved()
{
  // check to see if the mouse position is within the correlation view
  if ( (mouseY <= _correlation_view_origin[1]+_correlation_view_height) && (mouseX <= _correlation_view_origin[0]+_correlation_view_width+10) )
    _correlation_view.overView( mouseX-_correlation_view_origin[0], mouseY-_correlation_view_origin[1] );
  else if ( (mouseX <= _timeseries_view_origin[0]+_timeseries_view_width) && (mouseY <= _timeseries_view_origin[1]+_timeseries_view_height) )
    _timeseries_view.rolledOver( mouseX-_timeseries_view_origin[0], mouseY-_timeseries_view_origin[1] );
}

void mousePressed()
{
  // check to see if the mouse position is within the correlation view
  if ( (mouseX <= _timeseries_view_origin[0]) && (mouseY <= _correlation_view_origin[1]+_correlation_view_height) )
  {
    _correlation_view.pressedInView( mouseX-_correlation_view_origin[0], mouseY-_correlation_view_origin[1], _ctrl_pressed );
  }
  else
    _timeseries_view.pressedInView( mouseX-_timeseries_view_origin[0], mouseY-_timeseries_view_origin[1] );
}

void mouseReleased()
{
  _correlation_view.released();
}

void mouseDragged()
{
  _correlation_view.dragged( mouseX-_correlation_view_origin[0], mouseY-_correlation_view_origin[1] );
}

void keyPressed() 
{
  if ( key == CODED )
  {
    if ( keyCode == UP ) 
    { 
      _correlation_view.upKeyPressed();
    }
    else if ( keyCode == DOWN ) 
    { 
      _correlation_view.downKeyPressed();
    }
    else if ( keyCode == CONTROL ) _ctrl_pressed = true;
  }
  else if ( key == 'P' ) 
    _print_pdf = true;
  else if ( key == '1' ) { _correlation_view.numCorrToRender(1); _correlation_view.singleSelectedCorr(0); }
  else if ( key == '2' ) { _correlation_view.numCorrToRender(1); _correlation_view.singleSelectedCorr(1); }
  else if ( key == '3' ) { _correlation_view.numCorrToRender(1); _correlation_view.singleSelectedCorr(2); }
  else if ( key == '4' ) _correlation_view.numCorrToRender(2);
  else if ( key == '5' ) _correlation_view.numCorrToRender(3);
  
}

void keyReleased()
{ 
  if ( key == CODED )
  {
    if ( keyCode == CONTROL ) _ctrl_pressed = false;
  }
}


